@@ -25,7 +25,7 @@
 import sys
 import os
 
-from ducktools.lazyimporter import LazyImporter, ModuleImport
+from ducktools.lazyimporter import LazyImporter, ModuleImport, FromImport
 from ducktools.pythonfinder import list_python_installs
 
 _laz = LazyImporter(
@@ -33,8 +33,12 @@ _laz = LazyImporter(
         ModuleImport("argparse"),
         ModuleImport("csv"),
         ModuleImport("subprocess"),
-
-    ]
+        ModuleImport("platform"),
+        FromImport(".pythonorg_search", "PythonOrgSearch"),
+        FromImport("packaging.specifiers", "SpecifierSet"),
+        FromImport("urllib.error", "URLError"),
+    ],
+    globs=globals()
 )
 
 
@@ -76,40 +80,47 @@ def parse_args(args):
     )
     parser.add_argument("--min", help="Specify minimum Python version")
     parser.add_argument("--max", help="Specify maximum Python version")
-    parser.add_argument("--exact", help="Specify exact Python version")
+    parser.add_argument("--compatible", help="Specify compatible Python version")
+    parser.add_argument(
+        "--online",
+        action="store_true",
+        help="Get links to binaries from python.org (Windows only)"
+    )
+    parser.add_argument(
+        "--all-binaries",
+        action="store_true",
+        help="Provide *all* matching binaries and "
+             "not just the latest minor versions (Online only)"
+    )
+    parser.add_argument(
+        "--system",
+        action="store",
+        help="Get python installers for a different system (Windows, Darwin, Linux) (Online only)"
+    )
+    parser.add_argument(
+        "--machine",
+        action="store",
+        help="Get python installers for a different architecture (Windows: AMD64, ARM64, x86) (Online only)"
+    )
+
+    parser.add_argument(
+        "--prerelease",
+        action="store_true",
+        help="Include prerelease versions (Online only)"
+    )
 
     vals = parser.parse_args(args)
 
-    if vals.min:
-        min_ver = tuple(int(i) for i in vals.min.split("."))
-    else:
-        min_ver = None
-
-    if vals.max:
-        max_ver = tuple(int(i) for i in vals.max.split("."))
-    else:
-        max_ver = None
-
-    if vals.exact:
-        exact = tuple(int(i) for i in vals.exact.split("."))
-    else:
-        exact = None
-
-    return min_ver, max_ver, exact
+    return vals
 
 
-def main():
-    if sys.version_info < (3, 8):
-        v = sys.version_info
-        raise UnsupportedPythonError(
-            f"Python {v.major}.{v.minor}.{v.micro} is not supported. "
-            f"ducktools.pythonfinder requires Python 3.8 or later."
-        )
-
-    if sys.argv[1:]:
-        min_ver, max_ver, exact = parse_args(sys.argv[1:])
-    else:
-        min_ver, max_ver, exact = None, None, None
+def display_local_installs(min_ver=None, max_ver=None, compatible=None):
+    if min_ver:
+        min_ver = tuple(int(i) for i in min_ver.split("."))
+    if max_ver:
+        max_ver = tuple(int(i) for i in max_ver.split("."))
+    if compatible:
+        compatible = tuple(int(i) for i in compatible.split("."))
 
     installs = list_python_installs()
     headings = ["Python Version", "Executable Location"]
@@ -121,10 +132,11 @@ def main():
     print("Discoverable Python Installs")
     if sys.platform == "win32":
         print("+ - Listed in the Windows Registry ")
+        print("^ - This is a 32-bit Python install")
     if sys.platform != "win32":
-        print("[] - This python install is shadowed by another on Path")
-    print("* - This is the active python executable used to call this module")
-    print("** - This is the parent python executable of the venv used to call this module")
+        print("[] - This Python install is shadowed by another on Path")
+    print("* - This is the active Python executable used to call this module")
+    print("** - This is the parent Python executable of the venv used to call this module")
     print()
     print(headings_str)
     print(f"| {'-' * len(headings[0])} | {'-' * max_executable_len} |")
@@ -133,9 +145,9 @@ def main():
             continue
         elif max_ver and install.version > max_ver:
             continue
-        elif exact:
+        elif compatible:
             mismatch = False
-            for i, val in enumerate(exact):
+            for i, val in enumerate(compatible):
                 if val != install.version[i]:
                     mismatch = True
                     break
@@ -143,24 +155,100 @@ def main():
                 continue
 
         version_str = install.version_str
-        if install.shadowed:
-            version_str = f"[{version_str}]"
+
+        if sys.platform == "win32":
+            if install.metadata.get("InWindowsRegistry"):
+                version_str = f"+{version_str}"
+            if install.architecture == "32bit":
+                version_str = f"^{version_str}"
 
         if install.executable == sys.executable:
             version_str = f"*{version_str}"
-        elif sys.prefix != sys.base_prefix and install.executable.startswith(
-            sys.base_prefix
+        elif (
+            sys.prefix != sys.base_prefix
+            and os.path.commonpath([install.executable, sys.base_prefix]) == sys.base_prefix
         ):
             version_str = f"**{version_str}"
 
-        if sys.platform == "win32" and install.metadata.get("InWindowsRegistry"):
-            version_str = f"+{version_str}"
+        if install.shadowed:
+            version_str = f"[{version_str}]"
 
         print(f"| {version_str:>14s} | {install.executable:<{max_executable_len}s} |")
 
+
+def display_remote_binaries(
+        min_ver,
+        max_ver,
+        compatible,
+        all_binaries,
+        system,
+        machine,
+        prerelease
+):
+    specs = []
+    if min_ver:
+        specs.append(f">={min_ver}")
+    if max_ver:
+        specs.append(f"<{max_ver}")
+    if compatible:
+        specs.append(f"~={compatible}")
+
+    spec = _laz.SpecifierSet(",".join(specs))
+
+    searcher = _laz.PythonOrgSearch(system=system, machine=machine)
+    if all_binaries:
+        releases = searcher.all_matching_binaries(spec, prereleases=prerelease)
+    else:
+        releases = searcher.latest_minor_binaries(spec, prereleases=prerelease)
+
+    headings = ["Python Version", "URL"]
+    max_url_len = max(
+        len(headings[1]), max(len(release.url) for release in releases)
+    )
+    headings_str = f"| {headings[0]} | {headings[1]:<{max_url_len}s} |"
+
+    print(headings_str)
+    print(f"| {'-' * len(headings[0])} | {'-' * max_url_len} |")
+
+    for release in releases:
+        print(f"| {release.version:>14s} | {release.url:<{max_url_len}s} |")
+
+
+def main():
+    if sys.version_info < (3, 8):
+        v = sys.version_info
+        raise UnsupportedPythonError(
+            f"Python {v.major}.{v.minor}.{v.micro} is not supported. "
+            f"ducktools.pythonfinder requires Python 3.8 or later."
+        )
+
+    if sys.argv[1:]:
+        vals = parse_args(sys.argv[1:])
+        online = vals.online
+
+        if online:
+            system = vals.system if vals.system else _laz.platform.system()
+            machine = vals.machine if vals.machine else _laz.platform.machine()
+            try:
+                display_remote_binaries(
+                    vals.min,
+                    vals.max,
+                    vals.compatible,
+                    vals.all_binaries,
+                    system,
+                    machine,
+                    vals.prerelease,
+                )
+            except _laz.URLError:
+                print("Could not connect to python.org")
+        else:
+            display_local_installs(vals.min, vals.max, vals.compatible)
+    else:
+        # No arguments to parse
+        display_local_installs()
+
     stop_autoclose()
 
 
-
 if __name__ == "__main__":
     main()
